@@ -3,18 +3,13 @@ package etcdcertsigner
 import (
 	"bytes"
 	"context"
-	"crypto/x509"
-	"crypto/x509/pkix"
-	"errors"
 	"fmt"
-	"strings"
 	"time"
 
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
 	utilerrors "k8s.io/apimachinery/pkg/util/errors"
-	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/client-go/kubernetes"
 	corev1client "k8s.io/client-go/kubernetes/typed/core/v1"
 	corev1listers "k8s.io/client-go/listers/core/v1"
@@ -23,20 +18,13 @@ import (
 	configv1informers "github.com/openshift/client-go/config/informers/externalversions/config/v1"
 	configv1listers "github.com/openshift/client-go/config/listers/config/v1"
 	"github.com/openshift/library-go/pkg/controller/factory"
-	"github.com/openshift/library-go/pkg/crypto"
 	"github.com/openshift/library-go/pkg/operator/events"
 	"github.com/openshift/library-go/pkg/operator/resource/resourceapply"
 	"github.com/openshift/library-go/pkg/operator/v1helpers"
 
 	"github.com/openshift/cluster-etcd-operator/pkg/dnshelpers"
 	"github.com/openshift/cluster-etcd-operator/pkg/operator/operatorclient"
-)
-
-const (
-	EtcdCertValidity = 3 * 365 * 24 * time.Hour
-	peerOrg          = "system:etcd-peers"
-	serverOrg        = "system:etcd-servers"
-	metricOrg        = "system:etcd-metrics"
+	"github.com/openshift/cluster-etcd-operator/pkg/tlshelpers"
 )
 
 type EtcdCertSignerController struct {
@@ -127,43 +115,47 @@ func (c *EtcdCertSignerController) syncAllMasters(recorder events.Recorder) erro
 
 	// build the combined secrets that we're going to install
 	combinedPeerSecret := &corev1.Secret{
-		ObjectMeta: metav1.ObjectMeta{Namespace: operatorclient.TargetNamespace, Name: "etcd-all-peer"},
+		ObjectMeta: metav1.ObjectMeta{Namespace: operatorclient.TargetNamespace, Name: tlshelpers.EtcdAllPeerSecretName},
 		Type:       corev1.SecretTypeOpaque,
 		Data:       map[string][]byte{},
 	}
 	combinedServingSecret := &corev1.Secret{
-		ObjectMeta: metav1.ObjectMeta{Namespace: operatorclient.TargetNamespace, Name: "etcd-all-serving"},
+		ObjectMeta: metav1.ObjectMeta{Namespace: operatorclient.TargetNamespace, Name: tlshelpers.EtcdAllServingSecretName},
 		Type:       corev1.SecretTypeOpaque,
 		Data:       map[string][]byte{},
 	}
 	combinedServingMetricsSecret := &corev1.Secret{
-		ObjectMeta: metav1.ObjectMeta{Namespace: operatorclient.TargetNamespace, Name: "etcd-all-serving-metrics"},
+		ObjectMeta: metav1.ObjectMeta{Namespace: operatorclient.TargetNamespace, Name: tlshelpers.EtcdAllServingMetricsSecretName},
 		Type:       corev1.SecretTypeOpaque,
 		Data:       map[string][]byte{},
 	}
 	for _, node := range nodes {
-		currPeer, err := c.secretLister.Secrets(operatorclient.TargetNamespace).Get(getPeerClientSecretNameForNode(node))
+		peerSecretName := tlshelpers.GetPeerClientSecretNameForNode(node.Name)
+		servingSecretName := tlshelpers.GetServingSecretNameForNode(node.Name)
+		servingMetricsSecretName := tlshelpers.GetServingMetricsSecretNameForNode(node.Name)
+
+		currPeer, err := c.secretLister.Secrets(operatorclient.TargetNamespace).Get(peerSecretName)
 		if err != nil {
 			errs = append(errs, err)
 		} else {
-			combinedPeerSecret.Data[getPeerClientSecretNameForNode(node)+".crt"] = currPeer.Data["tls.crt"]
-			combinedPeerSecret.Data[getPeerClientSecretNameForNode(node)+".key"] = currPeer.Data["tls.key"]
+			combinedPeerSecret.Data[peerSecretName+".crt"] = currPeer.Data["tls.crt"]
+			combinedPeerSecret.Data[peerSecretName+".key"] = currPeer.Data["tls.key"]
 		}
 
-		currServing, err := c.secretLister.Secrets(operatorclient.TargetNamespace).Get(getServingSecretNameForNode(node))
+		currServing, err := c.secretLister.Secrets(operatorclient.TargetNamespace).Get(servingSecretName)
 		if err != nil {
 			errs = append(errs, err)
 		} else {
-			combinedServingSecret.Data[getServingSecretNameForNode(node)+".crt"] = currServing.Data["tls.crt"]
-			combinedServingSecret.Data[getServingSecretNameForNode(node)+".key"] = currServing.Data["tls.key"]
+			combinedServingSecret.Data[servingSecretName+".crt"] = currServing.Data["tls.crt"]
+			combinedServingSecret.Data[servingSecretName+".key"] = currServing.Data["tls.key"]
 		}
 
-		currServingMetrics, err := c.secretLister.Secrets(operatorclient.TargetNamespace).Get(getServingMetricsSecretNameForNode(node))
+		currServingMetrics, err := c.secretLister.Secrets(operatorclient.TargetNamespace).Get(servingMetricsSecretName)
 		if err != nil {
 			errs = append(errs, err)
 		} else {
-			combinedServingMetricsSecret.Data[getServingMetricsSecretNameForNode(node)+".crt"] = currServingMetrics.Data["tls.crt"]
-			combinedServingMetricsSecret.Data[getServingMetricsSecretNameForNode(node)+".key"] = currServingMetrics.Data["tls.key"]
+			combinedServingMetricsSecret.Data[servingMetricsSecretName+".crt"] = currServingMetrics.Data["tls.crt"]
+			combinedServingMetricsSecret.Data[servingMetricsSecretName+".key"] = currServingMetrics.Data["tls.key"]
 		}
 	}
 	if len(errs) > 0 {
@@ -187,20 +179,10 @@ func (c *EtcdCertSignerController) syncAllMasters(recorder events.Recorder) erro
 	return utilerrors.NewAggregate(errs)
 }
 
-func getPeerClientSecretNameForNode(node *corev1.Node) string {
-	return fmt.Sprintf("etcd-peer-%s", node.Name)
-}
-func getServingSecretNameForNode(node *corev1.Node) string {
-	return fmt.Sprintf("etcd-serving-%s", node.Name)
-}
-func getServingMetricsSecretNameForNode(node *corev1.Node) string {
-	return fmt.Sprintf("etcd-serving-metrics-%s", node.Name)
-}
-
 func (c *EtcdCertSignerController) createSecretForNode(node *corev1.Node, recorder events.Recorder) error {
-	etcdPeerClientCertName := getPeerClientSecretNameForNode(node)
-	etcdServingCertName := getServingSecretNameForNode(node)
-	metricsServingCertName := getServingMetricsSecretNameForNode(node)
+	etcdPeerClientCertName := tlshelpers.GetPeerClientSecretNameForNode(node.Name)
+	etcdServingCertName := tlshelpers.GetServingSecretNameForNode(node.Name)
+	metricsServingCertName := tlshelpers.GetServingMetricsSecretNameForNode(node.Name)
 
 	var err error
 	_, err = c.secretLister.Secrets(operatorclient.TargetNamespace).Get(etcdPeerClientCertName)
@@ -234,33 +216,19 @@ func (c *EtcdCertSignerController) createSecretForNode(node *corev1.Node, record
 	if err != nil {
 		return err
 	}
-	peerHostNames := append([]string{"localhost", etcdDiscoveryDomain}, nodeInternalIPs...)
-	serverHostNames := append([]string{
-		"localhost",
-		"etcd.kube-system.svc",
-		"etcd.kube-system.svc.cluster.local",
-		"etcd.openshift-etcd.svc",
-		"etcd.openshift-etcd.svc.cluster.local",
-		"*." + etcdDiscoveryDomain,
-		"127.0.0.1",
-		"::1",
-		"0:0:0:0:0:0:0:1",
-	}, nodeInternalIPs...)
-	// TODO debt left for @hexfusion or @sanchezl
-	fakePodFQDN := "etcd-client"
 
 	// create the certificates and update them in the API
-	pCert, pKey, err := createNewCombinedClientAndServingCerts(etcdCASecret.Data["tls.crt"], etcdCASecret.Data["tls.key"], fakePodFQDN, peerOrg, peerHostNames)
+	pCert, pKey, err := tlshelpers.CreatePeerCertKey(etcdCASecret.Data["tls.crt"], etcdCASecret.Data["tls.key"], append(nodeInternalIPs, etcdDiscoveryDomain))
 	err = c.createSecret(etcdPeerClientCertName, pCert, pKey, recorder)
 	if err != nil {
 		return err
 	}
-	sCert, sKey, err := createNewCombinedClientAndServingCerts(etcdCASecret.Data["tls.crt"], etcdCASecret.Data["tls.key"], fakePodFQDN, serverOrg, serverHostNames)
+	sCert, sKey, err := tlshelpers.CreateServerCertKey(etcdCASecret.Data["tls.crt"], etcdCASecret.Data["tls.key"], append(nodeInternalIPs, "*."+etcdDiscoveryDomain))
 	err = c.createSecret(etcdServingCertName, sCert, sKey, recorder)
 	if err != nil {
 		return err
 	}
-	metricCert, metricKey, err := createNewCombinedClientAndServingCerts(etcdMetricCASecret.Data["tls.crt"], etcdMetricCASecret.Data["tls.key"], fakePodFQDN, metricOrg, serverHostNames)
+	metricCert, metricKey, err := tlshelpers.CreateMetricCertKey(etcdMetricCASecret.Data["tls.crt"], etcdMetricCASecret.Data["tls.key"], nodeInternalIPs)
 	err = c.createSecret(metricsServingCertName, metricCert, metricKey, recorder)
 	if err != nil {
 		return err
@@ -269,74 +237,6 @@ func (c *EtcdCertSignerController) createSecretForNode(node *corev1.Node, record
 	return nil
 }
 
-func createNewCombinedClientAndServingCerts(caCert, caKey []byte, podFQDN, org string, peerHostNames []string) (*bytes.Buffer, *bytes.Buffer, error) {
-	cn, err := getCommonNameFromOrg(org)
-	etcdCAKeyPair, err := crypto.GetCAFromBytes(caCert, caKey)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	certConfig, err := etcdCAKeyPair.MakeServerCertForDuration(sets.NewString(peerHostNames...), EtcdCertValidity, func(cert *x509.Certificate) error {
-
-		cert.Issuer = pkix.Name{
-			OrganizationalUnit: []string{"openshift"},
-			CommonName:         cn,
-		}
-		cert.Subject = pkix.Name{
-			Organization: []string{org},
-			CommonName:   strings.TrimSuffix(org, "s") + ":" + podFQDN,
-		}
-		cert.ExtKeyUsage = []x509.ExtKeyUsage{x509.ExtKeyUsageClientAuth, x509.ExtKeyUsageServerAuth}
-
-		// TODO: Extended Key Usage:
-		// All profiles expect a x509.ExtKeyUsageCodeSigning set on extended Key Usages
-		// need to investigage: https://github.com/etcd-io/etcd/issues/9398#issuecomment-435340312
-		// TODO: some extensions are missing form cfssl.
-		// e.g.
-		//	X509v3 Subject Key Identifier:
-		//		B7:30:0B:CF:47:4E:21:AE:13:60:74:42:B0:D9:C4:F3:26:69:63:03
-		//	X509v3 Authority Key Identifier:
-		//		keyid:9B:C0:6B:0C:8E:5C:73:6A:83:B1:E4:54:97:D3:62:18:8A:9C:BC:1E
-		// TODO: Change serial number logic, to something as follows.
-		// The following is taken from CFSSL library.
-		// If CFSSL is providing the serial numbers, it makes
-		// sense to use the max supported size.
-
-		//	serialNumber := make([]byte, 20)
-		//	_, err = io.ReadFull(rand.Reader, serialNumber)
-		//	if err != nil {
-		//		return err
-		//	}
-		//
-		//	// SetBytes interprets buf as the bytes of a big-endian
-		//	// unsigned integer. The leading byte should be masked
-		//	// off to ensure it isn't negative.
-		//	serialNumber[0] &= 0x7F
-		//	cert.SerialNumber = new(big.Int).SetBytes(serialNumber)
-		return nil
-	})
-	if err != nil {
-		return nil, nil, err
-	}
-
-	certBytes := &bytes.Buffer{}
-	keyBytes := &bytes.Buffer{}
-	if err := certConfig.WriteCertConfig(certBytes, keyBytes); err != nil {
-		return nil, nil, err
-	}
-	return certBytes, keyBytes, nil
-}
-
-func getCommonNameFromOrg(org string) (string, error) {
-	if strings.Contains(org, "peer") || strings.Contains(org, "server") {
-		return "etcd-signer", nil
-	}
-	if strings.Contains(org, "metric") {
-		return "etcd-metric-signer", nil
-	}
-	return "", errors.New("unable to recognise secret name")
-}
-
 func (c *EtcdCertSignerController) getEtcdDiscoveryDomain() (string, error) {
 	infrastructure, err := c.infrastructureLister.Get("cluster")
 	if err != nil {
